@@ -1,8 +1,8 @@
 /*
-* 21/08/2004 - 19:29
+* 26/11/2006 - 17:02
 *
 * GraphMonkey - mono based graphing calculator
-* Copyright (C) 2004 Bellabes Lounis
+* Copyright (C) 2006 Lounis Bellabes
 * nolius@users.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@ class MainClass {
 	public static void Main(string[] args)
 	{
 		Application.Init ();
-		new MyWindow ();
+		new GraphMonkey ();
 		Application.Run ();
 	}
 }
