@@ -1,8 +1,8 @@
 /*
-* 21/08/2004 - 19:29
+* 26/11/2006 - 17:02
 *
 * GraphMonkey - mono based graphing calculator
-* Copyright (C) 2004 Bellabes Lounis
+* Copyright (C) 2006 Lounis Bellabes
 * nolius@users.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or
@@ -23,22 +23,18 @@
 using System;
 	
 
-
-	
-public class operation : object {
+public class operation : object{
 	public string phrase;
 	public string operateur;
 	public operation droite;
 	public operation gauche;
 
-	
 	public operation(string phrase) {
-		this.phrase=phrase;
-			
+		this.phrase=phrase;			
 	}
 		
 	public void decouper(){
-		//on recherche un + (le plus a droite possible) qui n'est pas dans une parenthese
+		//search a + (the more at rigth) not in Bracket
 		int position_plus=this.phrase.LastIndexOf("+");
 		bool trouver_bon_plus = false;
 		while (position_plus != -1 && trouver_bon_plus==false ){
@@ -53,7 +49,7 @@ public class operation : object {
 			position_plus = -1;
 		}
 		
-		//on recherche un - (le plus a droite possible) qui n'est pas dans une parenthese
+		//search a - (the more at rigth) not in Bracket
 		int position_moin=this.phrase.LastIndexOf("-");
 		bool trouver_bon_moin = false;
 		while (position_moin != -1 && trouver_bon_moin==false ){
@@ -68,9 +64,9 @@ public class operation : object {
 			position_moin = -1;
 		}
 			
-		// priorit au + et -
+		// priority for + and -
 			
-		// il y a d'abord un +
+		// initially  a +
 		if ( (position_plus!=-1 && position_moin==-1) || (position_plus!=-1 && position_moin!=-1 && position_plus>position_moin) ){
 			this.operateur = "+";
 			
@@ -82,7 +78,7 @@ public class operation : object {
 			phrase_droite = phrase_droite.Remove(0,position_plus+1);
 			this.droite = new operation(phrase_droite);				
 		}
-		// il y a d'abord un -
+		// initially  a -
 		else if ( (position_plus==-1 && position_moin!=-1) || (position_plus!=-1 && position_moin!=-1 && position_moin>position_plus) ){
 			this.operateur = "-";
 			
@@ -95,13 +91,10 @@ public class operation : object {
 			this.droite = new operation(phrase_droite);	
 	
 		}
-		
 			
-			
-			
-			// maintenant * et /
+			// now * et /
 		else {
-			//on recherche un * (le plus a doite possible) qui n'est pas dans une parenthese
+			//search a * (the more at rigth) not in Bracket
 			int position_multiplier=this.phrase.LastIndexOf("*");
 			bool trouver_bon_multiplier = false;
 			while (position_multiplier != -1 && trouver_bon_multiplier==false ){
@@ -116,7 +109,7 @@ public class operation : object {
 				position_multiplier= -1;
 			}
 				
-			//on recherche un / (le plus a doite possible) qui n'est pas dans une parenthese
+			//search a / (the more at rigth) not in Bracket
 			int position_diviser=this.phrase.LastIndexOf("/");
 			bool trouver_bon_diviser = false;
 			while (position_diviser != -1 && trouver_bon_diviser==false ){
@@ -132,10 +125,8 @@ public class operation : object {
 			}				
 			
 			
-			
-			
-			// il ya a d'abord un *
-			if ( (position_multiplier!=-1 && position_diviser==-1) || (position_multiplier!=-1 && position_diviser!=-1 && position_multiplier>position_diviser) ){ // il ya a d'abord un *
+			// initially  a *
+			if ( (position_multiplier!=-1 && position_diviser==-1) || (position_multiplier!=-1 && position_diviser!=-1 && position_multiplier>position_diviser) ){ // initially  a *
 				this.operateur = "*";
 					
 				string phrase_gauche=this.phrase;
@@ -146,8 +137,8 @@ public class operation : object {
 				phrase_droite = phrase_droite.Remove(0,position_multiplier+1);
 				this.droite = new operation(phrase_droite);				
 			}
-			// il y a d'abord un /
-			else if ( (position_multiplier==-1 && position_diviser!=-1) || (position_multiplier!=-1 && position_diviser!=-1 && position_diviser>position_multiplier) ){	// il ya a d'abord un /
+			// initially  a /
+			else if ( (position_multiplier==-1 && position_diviser!=-1) || (position_multiplier!=-1 && position_diviser!=-1 && position_diviser>position_multiplier) ){	// initially  a /
 				this.operateur = "/";
 				
 				string phrase_gauche=this.phrase;
@@ -160,10 +151,9 @@ public class operation : object {
 		
 			}
 			
-			// maintenant ^ (puissance) 
-			
+			// now ^
 			else{
-				//on recherche un ^ (le plus a gauche possible) qui n'est pas dans une parenthese
+				//search a ^ (the more at rigth) not in Bracket
 				int position_puissance=this.phrase.IndexOf("^");
 				bool trouver_bon_puissance = false;
 				while (position_puissance != -1 && trouver_bon_puissance==false ){
@@ -178,7 +168,7 @@ public class operation : object {
 					position_puissance= -1;
 				}				
 							
-				if ( (position_puissance!=-1) ){	// il y a un ^
+				if ( (position_puissance!=-1) ){	// there is a ^
 					this.operateur = "^";
 				
 					string phrase_gauche=this.phrase;
@@ -189,86 +179,86 @@ public class operation : object {
 					phrase_droite = phrase_droite.Remove(0,position_puissance+1);
 					this.droite = new operation(phrase_droite);				
 				}				
-				else{ // si il n'y a pas de + - / * ou ^ c'est que la phrase ressemble a "(2+x-3)" ou "2" ou "x" ou r(2*x)...
-			
-					//si phrase par une parenthese ex: (2+x-3) on supprime les parentheses pour pouvoir continuer le calcule
+				else{ // there is no + - / * or ^ so phrase is like "(2+x-3)" or "2" or "x" or r(2*x)...
+							
+					//if phrase is in bracket (2+x-3), delete bracket
 					if (this.phrase.StartsWith("(")){
 						this.operateur="()";
 					
 						string phrase_gauche=this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,1);	//on supprime le "(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin		
+						phrase_gauche = phrase_gauche.Remove(0,1);	//delete "(" in beginning 
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end		
 						this.gauche = new operation(phrase_gauche);
 					}
-					else if (this.phrase.StartsWith("sqrt(")){	// racine
+					else if (this.phrase.StartsWith("sqrt(")){	// root square
 						this.operateur = "sqrt()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "r(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "sqrt(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("cos(")){	// cos
 						this.operateur = "cos()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,4);	//on supprime le "cos(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "cos(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("sin(")){	// sin
 						this.operateur = "sin()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,4);	//on supprime le "sin(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "sin(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
-					else if (this.phrase.StartsWith("tan(")){	// sin
+					else if (this.phrase.StartsWith("tan(")){	// tan
 						this.operateur = "tan()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,4);	//on supprime le "tan(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "tan(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
-					else if (this.phrase.StartsWith("n(")){	// n
-						this.operateur = "n()";
+					else if (this.phrase.StartsWith("ln(")){	// ln
+						this.operateur = "ln()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,2);	//on supprime le "n(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,3);	//delete "ln(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("log(")){	// log
 						this.operateur = "log()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,4);	//on supprime le "log(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "log(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("acos(")){	// acos
 						this.operateur = "acos()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "acos(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "acos(" in beginningt
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("asin(")){	// asin
 						this.operateur = "asin()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "asin(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "asin(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("atan(")){	// atan
 						this.operateur = "atan()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "atan(" du debut
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "atan(" in beginning
 						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
 						this.gauche = new operation(phrase_gauche);
 					}
@@ -276,40 +266,53 @@ public class operation : object {
 						this.operateur = "cosh()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "cosh(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "cosh(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("sinh(")){	// sinh
 						this.operateur = "sinh()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "sinh(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "sinh(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
 					else if (this.phrase.StartsWith("tanh(")){	// tanh
 						this.operateur = "tanh()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,5);	//on supprime le "tanh(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,5);	//delete "tanh(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
-					else if (this.phrase.StartsWith("exp(")){	// tanh
+					else if (this.phrase.StartsWith("exp(")){	// exp
 						this.operateur = "exp()";
 						
 						string phrase_gauche = this.phrase;
-						phrase_gauche = phrase_gauche.Remove(0,4);	//on supprime le "exp(" du debut
-						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//on supprime le ")" de la fin				
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "exp(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
+						this.gauche = new operation(phrase_gauche);
+					}
+					else if (this.phrase.StartsWith("abs(")){	// abs
+						this.operateur = "abs()";
+						
+						string phrase_gauche = this.phrase;
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "abs(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
+						this.gauche = new operation(phrase_gauche);
+					}
+					else if (this.phrase.StartsWith("int(")){	// int
+						this.operateur = "int()";
+						
+						string phrase_gauche = this.phrase;
+						phrase_gauche = phrase_gauche.Remove(0,4);	//delete "int(" in beginning
+						phrase_gauche = phrase_gauche.Remove(phrase_gauche.Length-1,1);		//delete ")" in end				
 						this.gauche = new operation(phrase_gauche);
 					}
-					
-					
 				}
 			}
 		}
-		
 	}
 		
 		
@@ -319,7 +322,6 @@ public class operation : object {
 		}
 		
 		this.phrase = this.phrase.Replace("X", "x");
-		this.phrase = this.phrase.Replace(".", ".");
 		
 		this.phrase = this.phrase.Replace("(-", "(0-");
 		
@@ -349,16 +351,11 @@ public class operation : object {
 		this.phrase = this.phrase.Replace("9x", "9*x");
 			
 		this.phrase = this.phrase.Replace(")(", ")*(");
-		
-		
 	}
 		
 		
-		
-		
-		
-	public float calculer(float x){
-		float resultat=0;
+	public double calculer(double x){
+		double resultat=0;
 		
 		//this.corrige();
 		this.decouper();
@@ -382,197 +379,130 @@ public class operation : object {
 				return resultat;
 			}
 			else if(this.operateur=="^"){
-				// il faut faire une conversion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				double droite_double = Convert.ToDouble(this.droite.calculer(x));
-				resultat_double= Math.Pow(gauche_double,droite_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				double droite_double = this.droite.calculer(x);
+				resultat = Math.Pow(gauche_double,droite_double);
 				return resultat;
-				
 			}
-			
-			
 		}
-		// cas: () r() cos() sin() tan() et n()
+		// case: () r() cos() sin() tan() and n()
 		else if(this.gauche!=null){
 			if (this.operateur=="()"){
 				resultat=this.gauche.calculer(x);
 				return resultat;
 			}
 			else if (this.operateur=="sqrt()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Sqrt(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Sqrt(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="cos()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Cos(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Cos(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="sin()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Sin(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Sin(gauche_double);
 				return resultat;
 				
 			}
-			else if (this.operateur=="tan()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Tan(gauche_double);
-				resultat = (float) resultat_double;
-				
+			else if (this.operateur=="tan()"){	
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Tan(gauche_double);
 				return resultat;
 				
 			}
-			else if (this.operateur=="n()"){
-				resultat = 0-this.gauche.calculer(x);
+			else if (this.operateur=="ln()"){
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Log(gauche_double);
 				return resultat;
+				
 			}
 			else if (this.operateur=="log()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Log(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Log10(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="acos()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Acos(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Acos(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="asin()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Asin(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Asin(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="atan()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Atan(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Atan(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="cosh()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Cosh(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Cosh(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="sinh()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Sinh(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Sinh(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="tanh()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Tanh(gauche_double);
-				resultat = (float) resultat_double;
-				
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Tanh(gauche_double);
 				return resultat;
 				
 			}
 			else if (this.operateur=="exp()"){
-				// convertion
-				double resultat_double;	
-				double gauche_double = Convert.ToDouble(this.gauche.calculer(x));
-				resultat_double= Math.Exp(gauche_double);
-				resultat = (float) resultat_double;
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Exp(gauche_double);
+				return resultat;
 				
+			}
+			else if (this.operateur=="abs()"){
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Abs(gauche_double);
+				return resultat;
+				
+			}
+			else if (this.operateur=="int()"){
+				double gauche_double = this.gauche.calculer(x);
+				resultat = Math.Floor(gauche_double);
 				return resultat;
 				
 			}
 			
 			
+			
 		}
 		
-		else{ // phrase est obligatoirement de la forme "x" ou "2,1213" ou "2" 
+		else{ // phrase is like "x" or "2,1213" or "2" 
 			if (this.phrase=="x" || this.phrase=="X"){
 				return x;
 			}
 			else{
 				decimal res_d = Convert.ToDecimal(this.phrase);
-				float res_f = (float) res_d;	
+				double res_f = (double) res_d;	
 				resultat=res_f;
 				return resultat;
-		
 			}
 		}
-		return resultat;
-			
+		return resultat;	
 	}
-	
-	
+		
 
-		
-		
-		
-	// exemple: compte_depuis("a", 3) de "aaaaaa" =>3
-	public int compte_depuis(string ch, int debut){
-		int pos;
-		int nb=0;
-		bool arreter=false;
-		while( (debut <= this.phrase.Length) && (arreter == false) ){
-			pos=phrase.IndexOf(ch,debut);
-			if(pos != -1){
-				nb++;
-				debut=pos+1;
-			}
-			else{
-				arreter=true;
-			}
-			
-		}
-		return nb;
-	}
-		
-		
-		
-	// exemple: compte_jusqua("a", 3) de "aaaaaa" =>3
-	public int compte_jusqua(string ch, int fin){
+	// exemple: compte_jusqua("a", 3) for "aaaaaa" =>3
+	private int compte_jusqua(string ch, int fin){
 		int pos;
 		int nb=0;
 		bool arreter=false;
